from . import geometry
from . import material
from . import force
from . import analysis
from . import plot
from matplotlib import pyplot as plt
class model:
    def __init__(self,damGeometry,concrete,water,upliftForce,fig,ax):

        #First level

        self.damGeometry=damGeometry
        self.concrete=concrete
        self.water=water
        self.upliftForce=upliftForce
        self.fig=fig
        self.ax=ax



        #Second level

        self.sectionArea=geometry.sectionArea(damGeometry=self.damGeometry)
        self.sectionCentroid=geometry.sectionCentroid(damGeometry=self.damGeometry)
        self.upstreamWaterPressure=force.upstreamWaterPressure(damGeometry=self.damGeometry,water=self.water)
        self.downstreamWaterPressure=force.downstreamWaterPressure(damGeometry=self.damGeometry,water=self.water)
        self.plotDam=plot.plotDam(damGeometry=self.damGeometry,fig=self.fig,ax=self.ax)

        #Third level

        self.damGravity=force.damGravity(sectionArea=self.sectionArea,sectionCentroid=self.sectionCentroid,concrete=self.concrete)

        #Fourth level

        self.maxFriction=force.maxFriction(damGravity=self.damGravity,upstreamWaterPressure=self.upstreamWaterPressure,\
                                           downstreamWaterPressure=self.downstreamWaterPressure,upliftForce=self.upliftForce)
        self.overtuningFactor=analysis.overtuningFactor(damGeometry=self.damGeometry,upstreamWaterPressure=self.upstreamWaterPressure,\
                                                        downstreamWaterPressure=self.downstreamWaterPressure,upliftForce=self.upliftForce,\
                                                        damGravity=self.damGravity)

        #Fifth level

        self.slipFactor=analysis.slipFactor(upstreamWaterPressure=self.upstreamWaterPressure,downstreamWaterPressure=self.downstreamWaterPressure,\
                                            maxFriction=self.maxFriction)





import matplotlib.pyplot as plt
from .. import geometry
fig,ax=plt.subplots()
plt.close(fig)

class plotDam:
    def __init__(self,damGeometry,fig,ax):
        self.damPoints=damGeometry.damPoints
        self.fig=fig
        self.ax=ax
    def showDam(self):
        dummy = plt.figure()
        new_manager = dummy.canvas.manager
        new_manager.canvas.figure = self.fig
        self.fig.set_canvas(new_manager.canvas)
        self.ax.plot(*zip(*self.damPoints),color="darkorange")
        plt.show()


from .. import material
from .. import geometry
import numpy
import shapely
from shapely.geometry import Polygon


class upstreamWaterPressure:
    """
    Attributes
    ---------------------------------
    fx:Horizontal force of upstream pressure
    fy:Vertical force of upstream pressure
    xFocus:Centroid's x-coordinate
    yFocus:Centroid's y-coordinate
    """
    def __init__(self,damGeometry,water):
        if damGeometry.H>damGeometry.hu:
            self.fx=0.5*water.density*9.81*damGeometry.hu*damGeometry.hu
            self.fy=-0.5*water.density*9.81*damGeometry.hu*damGeometry.hu*(damGeometry.a/damGeometry.H)
            self.xFocus=(1/3)*damGeometry.a*damGeometry.hu/damGeometry.H
            self.yFocus=(1/3)*damGeometry.hu
        else:
            self.fx=0.5*water.density*9.81*damGeometry.hu*damGeometry.hu
            self.fy=-0.5*water.density*9.81*damGeometry.a*(2*damGeometry.hu-damGeometry.H)
            self.xFocus=(1/3)*damGeometry.a*(3*damGeometry.hu-2*damGeometry.H)/(2*damGeometry.hu-damGeometry.H)
            self.yFocus=(1/3)*damGeometry.hu

class downstreamWaterPressure:
    """
    Attributes
    ---------------------------------
    fx:Horizontal force of downstream pressure
    fy:Vertical force of downstream pressure
    xFocus:Centroid's x-coordinate
    yFocus:Centroid's y-coordinate
    **Assume that downstream water head is lower than h
    """
    def __init__(self,damGeometry,water):
        self.fx=-0.5*water.density*9.81*damGeometry.hd*damGeometry.hd
        self.fy=-0.5*water.density*9.81*damGeometry.hd*damGeometry.hd*(damGeometry.b/damGeometry.h)
        self.xFocus=damGeometry.a+damGeometry.l+damGeometry.b-(1/3)*damGeometry.b*damGeometry.hd/damGeometry.h
        self.yFocus=(1/3)*damGeometry.hd

class upliftForce:
    """
    Attributes
    ----------------------
    up:Tuples of the uplift pressure's shape
    upliftPolygon:Instance of shapely of up
    f:Magnitude of centroid uplift force
    x:Centroid force's x-center of uplift force
    """
    def __init__(self,upliftPressure,damGeometry):
        self.uP=upliftPressure+[(damGeometry.a+damGeometry.l+damGeometry.b,0),(0,0)]
        self.upliftPolygon = Polygon(self.uP)
        self.f=self.upliftPolygon.area
        self.x=self.upliftPolygon.centroid.coords[:][0][0]

class damGravity:
    '''
    Attributes
    --------------------------
    gravity:gravity of the dame
    x:Centroid's x-coordinate
    y:Centroid's y-coordinate
    '''
    def __init__(self,sectionArea,sectionCentroid,concrete):
        self.gravity=-1*9.81*sectionArea.area*concrete.density
        self.x=sectionCentroid.xCentroid
        self.y=sectionCentroid.yCentroid
class maxFriction:
    '''
    Attributes
    --------------------
    f:Friction on the dam's bottom face
    '''
    def __init__(self,damGravity,upstreamWaterPressure,downstreamWaterPressure,upliftForce):
        self.f=-1*numpy.tan(40/180*numpy.pi)*(-1*damGravity.gravity-upstreamWaterPressure.fy-downstreamWaterPressure.fy-upliftForce.f)





        





from .. import force
from .. import geometry


class slipFactor:
    '''
    Attributes
    sFactor:Slipping safety factor
    '''
    def __init__(self,upstreamWaterPressure,downstreamWaterPressure,maxFriction):
        self.sFactor=abs(maxFriction.f/(upstreamWaterPressure.fx+downstreamWaterPressure.fx))

class overtuningFactor:
    '''
    Attributes
    oFactor:Overturning safety factor
    '''
    def __init__(self,damGeometry,upstreamWaterPressure,downstreamWaterPressure,upliftForce,damGravity):
        self.oFactor=abs(
            (damGravity.gravity*(damGeometry.a+damGeometry.l+damGeometry.b-damGravity.x))/\
                        (upstreamWaterPressure.fx*upstreamWaterPressure.yFocus+\
                        upstreamWaterPressure.fy*(damGeometry.a+damGeometry.l+damGeometry.b-upstreamWaterPressure.xFocus)+\
                        upliftForce.f*(damGeometry.a+damGeometry.l+damGeometry.b-upliftForce.x)+\
                        downstreamWaterPressure.fy*(damGeometry.a+damGeometry.l+damGeometry.b-downstreamWaterPressure.xFocus)+\
                        downstreamWaterPressure.fx*downstreamWaterPressure.yFocus)
        )




from . import damGeometry


class sectionArea:
    def __init__(self,damGeometry):
        '''
        Attributes
        --------------------
        area:Area of the dam's section
        '''
        self.area=0.5*damGeometry.a*damGeometry.H+0.5*damGeometry.b*damGeometry.h+damGeometry.l*(damGeometry.h+damGeometry.c)

class sectionCentroid:
    def __init__(self,damGeometry):
        '''
        Attributes
        ---------------------------------
        xcentroid:x-coordinate of dam's shape center
        yCentroid:y-coordinate of dam's shape center
        '''
        self.xCentroid=(0.5*damGeometry.a*damGeometry.H*(2/3)*damGeometry.a+
                        damGeometry.l * (damGeometry.h+damGeometry.c)*(damGeometry.a+0.5*damGeometry.l)+
                        0.5*damGeometry.b*damGeometry.h*(damGeometry.a+damGeometry.l+(1/3)*damGeometry.b))/\
                       (0.5*damGeometry.a*damGeometry.H+0.5*damGeometry.b*damGeometry.h+damGeometry.l*(damGeometry.h+damGeometry.c))
        self.yCentroid=(0.5*damGeometry.a*damGeometry.H*(1/3)*damGeometry.H+
                        damGeometry.l * damGeometry.H*(0.5*damGeometry.h+0.5*damGeometry.c)+
                        0.5*damGeometry.b*damGeometry.h*((1/3)*damGeometry.h))/\
                       (0.5*damGeometry.a*damGeometry.H+0.5*damGeometry.b*damGeometry.h+damGeometry.l*(damGeometry.h+damGeometry.c))




class concrete:
    '''
    Attributes
    -----------------
    Desnsity:Density of the concrete
    '''
    def __init__(self,density):
        self.density=density
class water:
    '''
    Attributes
    ------------------
    Density:Density of the water
    '''
    def __init__(self,density):
        self.density=density



class damGeometry:
    '''
    Attributes
    --------------
    H:Dam height of upstream face
    h:Dam height of dowstream face
    l:Length of crest
    a:Horizontal projection of the upstream face
    b:Horizontal projection of the downstream face
    c:Height of the downstream crest
    hu:Upstream water's height
    hd:Downstream water's height
    '''
    def __init__(self,H,h,l,a,b,c,hu,hd):
        self.H=H
        self.h=h
        self.l=l
        self.a=a
        self.b=b
        self.c=c
        self.hu=hu
        self.hd=hd

        self.damPoints=[
            (0,0),
            (self.a+self.l+self.b,0),
            (self.a+self.l,self.h),
            (self.a+self.l,self.h+self.c),
            (self.a,self.h+self.c),
            (self.a,self.H),
            (0,0)
            ]

